# JFrog Xray HA on Kubernetes Helm Chart

## Prerequisites Details

* Kubernetes 1.8+

## Chart Details
This chart will do the following:

* Optionally deploy PostgreSQL, MongoDB
* Deploy RabbitMQ (optionally as an HA cluster)
* Deploy JFrog Xray micro-services

## Requirements
- A running Kubernetes cluster
  - Dynamic storage provisioning enabled
  - Default StorageClass set to allow services using the default StorageClass for persistent storage
- A running Artifactory
- [Kubectl](https://kubernetes.io/docs/tasks/tools/install-kubectl/) installed and setup to use the cluster
- [Helm](https://helm.sh/) v2 or v3 installed

## Install JFrog Xray

### Add JFrog Helm repository
Before installing JFrog helm charts, you need to add the [JFrog helm repository](https://charts.jfrog.io/) to your helm client
```bash
helm repo add jfrog https://charts.jfrog.io
```

### Install Chart
Install JFrog Xray
```bash
helm upgrade --install xray --namespace xray jfrog/xray --version 2.0.2
```

## Status
See the status of your deployed **helm** releases
```bash
helm status xray
```

## Upgrade
To upgrade an existing Xray, you still use **helm**
```bash
# Update existing deployed version to 2.1.2
helm upgrade --set common.xrayVersion=2.1.2 jfrog/xray
```

If Xray was installed without providing a value to postgresql.postgresqlPassword (a password was autogenerated), follow these instructions:
1. Get the current password by running:
```bash
POSTGRES_PASSWORD=$(kubectl get secret -n <namespace> <myrelease>-postgresql -o jsonpath="{.data.postgresql-password}" | base64 --decode)
```
2. Upgrade the release by passing the previously auto-generated secret:
```bash
helm upgrade <myrelease> jfrog/xray --set postgresql.postgresqlPassword=${POSTGRES_PASSWORD}
```

If Xray was installed without providing a value to rabbitmq.rabbitmqPassword/rabbitmq-ha.rabbitmqPassword (a password was autogenerated), follow these instructions:
1. Get the current password by running:
```bash
RABBITMQ_PASSWORD=$(kubectl get secret -n <namespace> <myrelease>-rabbitmq -o jsonpath="{.data.rabbitmq-password}" | base64 --decode)
```
2. Upgrade the release by passing the previously auto-generated secret:
```bash
helm upgrade <myrelease> jfrog/xray --set rabbitmq.rabbitmqPassword=${RABBITMQ_PASSWORD}/rabbitmq-ha.rabbitmqPassword=${RABBITMQ_PASSWORD}
```

If Xray was installed without providing a value to mongodb.mongodbPassword and mongodb.mongodbRootPassword (a password was autogenerated), follow these instructions:
1. Get the current password by running:
```bash
MONGODB_ROOT_PASSWORD=$(kubectl get secret -n <namespace> <myrelease>-mongodb -o jsonpath="{.data.mongodb-root-password}" | base64 --decode)
MONGODB_PASSWORD=$(kubectl get secret -n <namespace> <myrelease>-mongodb -o jsonpath="{.data.mongodb-password}" | base64 --decode)
```
2. Upgrade the release by passing the previously auto-generated secret:
```bash
helm upgrade <myrelease> jfrog/xray --set mongodb.mongodbRootPassword=${MONGODB_ROOT_PASSWORD} --set mongodb.mongodbPassword=${MONGODB_PASSWORD}
```

If Xray was installed with all of the default values (e.g. with no user-provided values for mongo/rabbit/postgres), follow these steps:
1. Retrieve all current passwords (rabbitmq/postgresql/mongodb) as explained in the above section.
2. Upgrade the release by passing the previously auto-generated secrets:
```bash
helm upgrade --name xray jfrog/xray \
    --set mongodb.mongodbRootPassword=${MONGODB_ROOT_PASSWORD} \
    --set mongodb.mongodbPassword=${MONGODB_PASSWORD} \
    --set rabbitmq-ha.rabbitmqPassword=${RABBITMQ_PASSWORD} \
    --set postgresql.postgresqlPassword=${POSTGRES_PASSWORD}
```

## Remove
Removing a **helm** release is done with
```bash
# Remove the Xray services and data tools
helm delete --purge xray

# Remove the data disks
kubectl delete pvc -l release=xray
```

### Deploying Xray for small/medium/large instllations
In the chart directory, we have added three values files, one for each installation type - small/medium/large. These values files are recommendations for setting resources requests and limits for your installation. The values are derived from the following [documentation](https://www.jfrog.com/confluence/display/EP/Installing+on+Kubernetes#InstallingonKubernetes-Systemrequirements). You can find them in the corresponding chart directory -  values-small.yaml, values-medium.yaml and values-large.yaml

### Create a unique Master Key
JFrog Xray requires a unique master key to be used by all micro-services in the same cluster. By default the chart has one set in values.yaml (`common.masterKey`).

**This key is for demo purpose and should not be used in a production environment!**

You should generate a unique one and pass it to the template at install/upgrade time.
```bash
# Create a key
export MASTER_KEY=$(openssl rand -hex 32)
echo ${MASTER_KEY}

# Pass the created master key to helm
helm install --set common.masterKey=${MASTER_KEY} -n xray jfrog/xray
```
**NOTE:** Make sure to pass the same master key with `--set common.masterKey=${MASTER_KEY}` on all future calls to `helm install` and `helm upgrade`!

## Special deployments
This is a list of special use cases for non-standard deployments

### High Availability
For **high availability** of Xray, set the replica count per service be equal or higher than **2**. Recommended is **3**.
> It is highly recommended to also set **RabbitMQ** to run as an HA cluster.
```bash
# Start Xray with 3 replicas per service and 3 replicas for RabbitMQ
helm install -n xray --set analysis.replicaCount=3,server.replicaCount=3,indexer.replicaCount=3,persist.replicaCount=3,rabbitmq-ha.replicaCount=3 jfrog/xray
```

### External Databases

**For production grade installations it is recommended to use an external PostgreSQL and MongoDB with a static password**

There is an option to use external database services (MongoDB or PostgreSQL) for your Xray.

#### MongoDB
To use an external **MongoDB**, You need to set Xray **MongoDB** connection URL.

For this, pass the parameter: `mongodb.enabled=false` and `global.mongoUrl=${XRAY_MONGODB_CONN_URL}`.

**IMPORTANT:** Make sure the DB is already created before deploying Xray services
```bash
# Passing a custom MongoDB to Xray

# Example
# MongoDB host: custom-mongodb.local
# MongoDB port: 27017
# MongoDB user: xray
# MongoDB password: password1_X

export XRAY_MONGODB_CONN_URL='mongodb://${MONGODB_USER}:${MONGODB_PASSWORD}@custom-mongodb.local:27017/?authSource=${MONGODB_DATABASE}&authMechanism=SCRAM-SHA-1'
helm install -n xray \
    --set mongodb.enabled=false \
    --set global.mongoUrl="${XRAY_MONGODB_CONN_URL}" \
    jfrog/xray
```

#### PostgreSQL

##### PostgreSQL without TLS
To use an external **PostgreSQL**, you need to disable the use of the bundled **PostgreSQL** and set a custom **PostgreSQL** connection URL.

For this, pass the parameters: `postgresql.enabled=false` and `global.postgresqlUrl=${XRAY_POSTGRESQL_CONN_URL}`.

**IMPORTANT:** Make sure the DB is already created before deploying Xray services
```bash
# Passing a custom PostgreSQL to Xray

# Example
export POSTGRESQL_HOST=custom-postgresql-host
export POSTGRESQL_PORT=5432
export POSTGRESQL_USER=xray
export POSTGRESQL_PASSWORD=password2_X
export POSTGRESQL_DATABASE=xraydb

export XRAY_POSTGRESQL_CONN_URL="postgres://${POSTGRESQL_USER}:${POSTGRESQL_PASSWORD}@${POSTGRESQL_HOST}:${POSTGRESQL_PORT}/${POSTGRESQL_DATABASE}?sslmode=disable"
helm install -n xray \
    --set postgresql.enabled=false \
    --set global.postgresqlUrl="${XRAY_POSTGRESQL_CONN_URL}" \
    jfrog/xray
```

##### PostgreSQL with TLS
If external **PostgreSQL** is set with TLS, you need to disable the use of the bundled **PostgreSQL**, set a custom **PostgreSQL** connection URL and provide a secret with **PostgreSQL** TLS certificates.

Create the Kubernetes secret (assuming the local files are `client-cert.pem	client-key.pem server-ca.pem`)
```bash
kubectl create secret generic postgres-tls --from-file=client-key.pem --from-file=client-cert.pem --from-file=server-ca.pem
```

**IMPORTANT:** `PostgreSQL` connection URL needs to have listed TLS files with the path `/var/opt/jfrog/xray/data/tls/` 
and `sslmode==verify-ca` otherwise Xray will fail to connect to Postgres.

```bash
# Passing a custom PostgreSQL with TLS to Xray

# Example
export POSTGRESQL_HOST=custom-postgresql-host
export POSTGRESQL_PORT=5432
export POSTGRESQL_USER=xray
export POSTGRESQL_PASSWORD=password2_X
export POSTGRESQL_DATABASE=xraydb
export POSTGRESQL_SERVER_CA=server-ca.pem
export POSTGRESQL_CLIENT_CERT=client-key.pem
export POSTGRESQL_CLIENT_KEY=client-cert.pem
export POSTGRESQL_TLS_SECRET=postgres-tls

export XRAY_POSTGRESQL_CONN_URL="postgres://${POSTGRESQL_USER}:${POSTGRESQL_PASSWORD}@${POSTGRESQL_HOST}:${POSTGRESQL_PORT}/${POSTGRESQL_DATABASE}?sslrootcert=/var/opt/jfrog/xray/data/tls/${POSTGRESQL_SERVER_CA}&sslkey=/var/opt/jfrog/xray/data/tls/${POSTGRESQL_CLIENT_KEY}&sslcert=/var/opt/jfrog/xray/data/tls/${POSTGRESQL_CLIENT_CERT}&sslmode=verify-ca"
helm install -n xray \
    --set postgresql.enabled=false \
    --set global.postgresqlUrl="${XRAY_POSTGRESQL_CONN_URL}" \
    --set global.postgresqlTlsSecret=${POSTGRESQL_TLS_SECRET}
    jfrog/xray
```

### Logger sidecars
This chart provides the option to add sidecars to tail various logs from Xray containers. See the available values in `values.yaml`

  Get list of containers in the pod
```bash
kubectl get pods -n <NAMESPACE> <POD_NAME> -o jsonpath='{.spec.containers[*].name}' | tr ' ' '\n'
```

  View specific log
```bash
kubectl logs -n <NAMESPACE> <POD_NAME> -c <LOG_CONTAINER_NAME>
```

### Custom init containers
There are cases where a special, unsupported init processes is needed like checking something on the file system or testing something before spinning up the main container.

For this, there is a section for writing custom init containers before and after the predefined init containers in the [values.yaml](values.yaml) . By default it's commented out
```yaml
common:
  ## Add custom init containers executed before predefined init containers
  customInitContainersBegin: |
    ## Init containers template goes here ##

    ## Add custom init containers executed after predefined init containers
  customInitContainers: |
    ## Init containers template goes here ##
```

## Configuration
The following table lists the configurable parameters of the xray chart and their default values.

|         Parameter            |                    Description                   |           Default                  |
|------------------------------|--------------------------------------------------|------------------------------------|
| `imagePullSecrets`           | Docker registry pull secret                      |                                    |
| `imagePullPolicy`            | Container pull policy                            | `IfNotPresent`                     |
| `initContainerImage`         | Init container image                             | `alpine:3.6`                       |
| `serviceAccount.create`   | Specifies whether a ServiceAccount should be created| `true`                             |
| `serviceAccount.name`     | The name of the ServiceAccount to create            | Generated using the fullname template |
| `rbac.create`             | Specifies whether RBAC resources should be created  | `true`                             |
| `rbac.role.rules`         | Rules to create                                     | `[]`                               |
| `ingress.enabled`            | If true, Xray Ingress will be created            | `false`                            |
| `ingress.annotations`        | Xray Ingress annotations                         | `{}`                               |
| `ingress.hosts`              | Xray Ingress hostnames                           | `[]`                               |
| `ingress.tls`                | Xray Ingress TLS configuration (YAML)            | `[]`                               |
| `ingress.defaultBackend.enabled` | If true, the default `backend` will be added using serviceName and servicePort | `true` |
| `ingress.labels`              | Xray Ingress labels                             | `{}`                               |
| `ingress.additionalRules`              | Xray Ingress addtional rules                             | `{}`                               |
| `postgresql.enabled`              | Use enclosed PostgreSQL as database         | `true`                             |
| `postgresql.postgresqlDatabase`     | PostgreSQL database name                    | `xraydb`                           |
| `postgresql.postgresqlUser`         | PostgreSQL database user                    | `xray`                             |
| `postgresql.postgresqlPassword`     | PostgreSQL database password                | ` `                                |
| `postgresql.postgresqlExtendedConf.maxConnections`  | PostgreSQL max_connections parameter | `500`                           |
| `postgresql.persistence.enabled`  | PostgreSQL use persistent storage           | `true`                             |
| `postgresql.persistence.size`     | PostgreSQL persistent storage size          | `50Gi`                             |
| `postgresql.persistence.existingClaim`  | PostgreSQL use existing persistent storage          | ` `                  |
| `postgresql.service.port`         | PostgreSQL database port                    | `5432`                             |
| `postgresql.resources.requests.memory`    | PostgreSQL initial memory request   |                                    |
| `postgresql.resources.requests.cpu`       | PostgreSQL initial cpu request      |                                    |
| `postgresql.resources.limits.memory`      | PostgreSQL memory limit             |                                    |
| `postgresql.resources.limits.cpu`         | PostgreSQL cpu limit                |                                    |
| `postgresql.nodeSelector`                 | PostgreSQL node selector            | `{}`                               |
| `postgresql.affinity`                     | PostgreSQL node affinity            | `{}`                               |
| `postgresql.tolerations`                  | PostgreSQL node tolerations         | `[]`                               |
| `mongodb.enabled`                         | Enable Mongodb                      | `true`                             |
| `mongodb.image.tag`                       | Mongodb docker image tag            | `3.6.3`                            |
| `mongodb.image.pullPolicy`                | Mongodb Container pull policy       | `IfNotPresent`                     |
| `mongodb.persistence.enabled`    | Mongodb persistence volume enabled           | `true`                             |
| `mongodb.persistence.existingClaim`   | Use an existing PVC to persist data               | `nil`                    |
| `mongodb.persistence.storageClass`    | Storage class of backing PVC                      | `generic`                |
| `mongodb.persistence.size`            | Mongodb persistence volume size             | `50Gi`                         |
| `mongodb.livenessProbe.initialDelaySeconds` | Mongodb delay before liveness probe is initiated   | ` `               |
| `mongodb.readinessProbe.initialDelaySeconds`| Mongodb delay before readiness probe is initiated  | ` `               |
| `mongodb.mongodbExtraFlags`                 | MongoDB additional command line flags | `["--wiredTigerCacheSizeGB=1"]`|
| `mongodb.mongodbDatabase`                   | Mongodb Database for Xray                       | `xray`               |
| `mongodb.mongodbRootPassword`                  | Mongodb Database Password for root user      | ` `                  |
| `mongodb.mongodbUsername`                      | Mongodb Database Xray User                   | `admin`              |
| `mongodb.mongodbPassword`                      | Mongodb Database Password for Xray User      | ` `                  |
| `mongodb.nodeSelector`                         | Mongodb node selector                        | `{}`                 |
| `mongodb.affinity`                             | Mongodb node affinity                        | `{}`                 |
| `mongodb.tolerations`                          | Mongodb node tolerations                     | `[]`                 |
| `rabbitmq.enabled`                             | RabbitMQ enabled uses rabbitmq               | `false`              |
| `rabbitmq.replicas`                            | RabbitMQ replica count               | `1`              |
| `rabbitmq.rbacEnabled`                         | If true, create & use RBAC resources         | `true`               |
| `rabbitmq.rabbitmq.username`                    | RabbitMQ application username                | `guest`               |
| `rabbitmq.rabbitmq.password`                    | RabbitMQ application password                |                |
| `rabbitmq.rabbitmq.existingPasswordSecret`      | RabbitMQ existingPasswordSecret               |                |
| `rabbitmq.rabbitmq.erlangCookie`                | RabbitMQ Erlang cookie                       | `XRAYRABBITMQCLUSTER`|
| `rabbitmq.service.nodePort`                    | RabbitMQ node port                           | `5672`               |
| `rabbitmq.persistence.enabled`            | If `true`, persistent volume claims are created | `true`            |
| `rabbitmq.persistence.accessMode`            | RabbitMQ persistent volume claims access mode | `ReadWriteOnce`            |
| `rabbitmq.persistence.size`               | RabbitMQ Persistent volume size              | `20Gi`               |
| `rabbitmq-ha.enabled`                          | RabbitMQ enabled uses rabbitmq-ha            | `true`               |
| `rabbitmq-ha.replicaCount`                     | RabbitMQ Number of replica                   | `1`                  |
| `rabbitmq-ha.rabbitmqUsername`                 | RabbitMQ application username                | `guest`              |
| `rabbitmq-ha.rabbitmqPassword`                 | RabbitMQ application password                | ` `                  |
| `rabbitmq-ha.existingSecret`                   | RabbitMQ existingSecret                      | ` `                  |
| `rabbitmq-ha.rabbitmqErlangCookie`             | RabbitMQ Erlang cookie                       | `XRAYRABBITMQCLUSTER`|
| `rabbitmq-ha.rabbitmqMemoryHighWatermark`      | RabbitMQ Memory high watermark               | `500MB`              |
| `rabbitmq-ha.persistentVolume.enabled`         | If `true`, persistent volume claims are created | `true`            |
| `rabbitmq-ha.persistentVolume.size`            | RabbitMQ Persistent volume size              | `20Gi`               |
| `rabbitmq-ha.rbac.create`                      | If true, create & use RBAC resources         | `true`               |
| `rabbitmq-ha.nodeSelector`                     | RabbitMQ node selector                       | `{}`                 |
| `rabbitmq-ha.tolerations`                      | RabbitMQ node tolerations                    | `[]`                 |
| `logger.image.repository`                      | Repository for logger image                  | `busybox`            |
| `logger.image.tag`                             | Tag for logger image                         | `1.30`               |
| `common.xrayVersion`                           | Xray image tag                               | `.Chart.AppVersion`  |
| `common.xrayConfigPath`                        | Xray config path                             | `/var/opt/jfrog/xray/data`     |
| `common.preStartCommand`                       | Xray Custom command to run before startup. Runs BEFORE any microservice-specific preStartCommand |     |
| `common.xrayUserId`                            | Xray User Id                                 | `1035`               |
| `common.xrayGroupId`                           | Xray Group Id                                | `1035`               |
| `common.masterKey`  | Xray Master Key Can be generated with `openssl rand -hex 32` | `FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF` |
| `common.masterKeySecretName`  | Name of precreated secret which contains xray masterKey, key for secret must contain `master-key`  | ` ` |
| `common.customInitContainersBegin`             | Custom init containers to run before existing init containers                       | ` `                  |
| `common.customInitContainers`                  | Custom init containers to run after existing init containers                       | ` `                  |
| `common.xrayConfig`                            | Additional xray yaml configuration to be written to xray_config.yaml file                       | ``                  |
| `global.mongoUrl`                              | Xray external MongoDB URL                    | ` `                  |
| `global.postgresqlUrl`                         | Xray external PostgreSQL URL                 | ` `                  |
| `global.postgresqlTlsSecret`                   | Xray external PostgreSQL TLS files secret    | ` `                  |
| `analysis.name`                                | Xray Analysis name                           | `xray-analysis`      |
| `analysis.image`                               | Xray Analysis container image                | `docker.bintray.io/jfrog/xray-analysis` |
| `analysis.replicaCount`                        | Xray Analysis replica count                  | `1`                  |
| `analysis.updateStrategy`                      | Xray Analysis update strategy                | `RollingUpdate`      |
| `analysis.podManagementPolicy`                 | Xray Analysis pod management policy          | `Parallel`           |
| `analysis.internalPort`                        | Xray Analysis internal port                  | `7000`               |
| `analysis.externalPort`                        | Xray Analysis external port                  | `7000`               |
| `analysis.service.type`                        | Xray Analysis service type                   | `ClusterIP`          |
| `analysis.livenessProbe.enabled`               | Xray Analysis liveness probe enabled                       | `true`                  |
| `analysis.livenessProbe.path`                  | Xray Analysis liveness probe path                       | `/debug/pprof/`                  |
| `analysis.livenessProbe.initialDelaySeconds`   | Xray Analysis delay before liveness probe is initiated                       | `90`                  |
| `analysis.livenessProbe.periodSeconds`         | Xray Analysis how often to perform the probe                       | `10`                  |
| `analysis.livenessProbe.timeoutSeconds`        | Xray Analysis when the probe times out                       | `1`                  |
| `analysis.livenessProbe.failureThreshold`      | Xray Analysis minimum consecutive failures for the probe to be considered failed after having succeeded        | `3`           |
| `analysis.livenessProbe.successThreshold`      | Xray Analysis minimum consecutive successes for the probe to be considered successful after having failed | `1`                  |
| `analysis.readinessProbe.enabled`               | Xray Analysis readiness probe enabled                       | `true`                  |
| `analysis.readinessProbe.path`                  | Xray Analysis readiness probe path                       | `/debug/pprof/`                  |
| `analysis.readinessProbe.initialDelaySeconds`   | Xray Analysis delay before readiness probe is initiated                       | `90`                  |
| `analysis.readinessProbe.periodSeconds`         | Xray Analysis how often to perform the probe                       | `10`                  |
| `analysis.readinessProbe.timeoutSeconds`        | Xray Analysis when the probe times out                       | `1`                  |
| `analysis.readinessProbe.failureThreshold`      | Xray Analysis minimum consecutive failures for the probe to be considered failed after having succeeded        | `3`           |
| `analysis.readinessProbe.successThreshold`      | Xray Analysis minimum consecutive successes for the probe to be considered successful after having failed | `1`                  |
| `analysis.persistence.size`                    | Xray Analysis storage size limit             | `10Gi`               |
| `analysis.resources`                           | Xray Analysis resources                      | `{}`                 |
| `analysis.preStartCommand`                     | Xray Analysis Custom command to run before startup. Runs AFTER the `common.preStartCommand` |     |
| `analysis.loggers`                             | Xray Analysis loggers (see values.yaml for possible values)  | ` `  |
| `analysis.loggersResources.requests.memory`    | Xray Analysis loggers initial memory request                        |               |
| `analysis.loggersResources.requests.cpu`       | Xray Analysis loggers initial cpu request                           |               |
| `analysis.loggersResources.limits.memory`      | Xray Analysis loggers memory limit                                  |               |
| `analysis.loggersResources.limits.cpu`         | Xray Analysis loggers cpu limit                                     |               |
| `analysis.nodeSelector`                        | Xray Analysis node selector                  | `{}`                 |
| `analysis.affinity`                            | Xray Analysis node affinity                  | `{}`                 |
| `analysis.tolerations`                         | Xray Analysis node tolerations               | `[]`                 |
| `analysis.annotations`                          | Xray Analysis annotations                     | `{}`                               |
| `indexer.name`                                 | Xray Indexer name                            | `xray-indexer`       |
| `indexer.image`                                | Xray Indexer container image                 | `docker.bintray.io/jfrog/xray-indexer`  |
| `indexer.replicaCount`                         | Xray Indexer replica count                   | `1`                  |
| `indexer.annotations`                          | Xray Indexer annotations                     | `{}`                               |
| `indexer.updateStrategy`                       | Xray Indexer update strategy                 | `RollingUpdate`      |
| `indexer.podManagementPolicy`                  | Xray Indexer pod management policy           | `Parallel`           |
| `indexer.internalPort`                         | Xray Indexer internal port                   | `7002`               |
| `indexer.externalPort`                         | Xray Indexer external port                   | `7002`               |
| `indexer.service.type`                         | Xray Indexer service type                    | `ClusterIP`          |
| `indexer.livenessProbe.enabled`               | Xray Indexer liveness probe enabled                       | `true`                  |
| `indexer.livenessProbe.path`                  | Xray Indexer liveness probe path                       | `/debug/pprof/`                  |
| `indexer.livenessProbe.initialDelaySeconds`   | Xray Indexer delay before liveness probe is initiated                       | `90`                  |
| `indexer.livenessProbe.periodSeconds`         | Xray Indexer how often to perform the probe                       | `10`                  |
| `indexer.livenessProbe.timeoutSeconds`        | Xray Indexer when the probe times out                       | `1`                  |
| `indexer.livenessProbe.failureThreshold`      | Xray Indexer minimum consecutive failures for the probe to be considered failed after having succeeded        | `3`           |
| `indexer.livenessProbe.successThreshold`      | Xray Indexer minimum consecutive successes for the probe to be considered successful after having failed | `1`                  |
| `indexer.readinessProbe.enabled`               | Xray Indexer readiness probe enabled                       | `true`                  |
| `indexer.readinessProbe.path`                  | Xray Indexer readiness probe path                       | `/debug/pprof/`                  |
| `indexer.readinessProbe.initialDelaySeconds`   | Xray Indexer delay before readiness probe is initiated                       | `90`                  |
| `indexer.readinessProbe.periodSeconds`         | Xray Indexer how often to perform the probe                       | `10`                  |
| `indexer.readinessProbe.timeoutSeconds`        | Xray Indexer when the probe times out                       | `1`                  |
| `indexer.readinessProbe.failureThreshold`      | Xray Indexer minimum consecutive failures for the probe to be considered failed after having succeeded        | `3`           |
| `indexer.readinessProbe.successThreshold`      | Xray Indexer minimum consecutive successes for the probe to be considered successful after having failed | `1`                  |
| `indexer.customVolumes`                         | Custom volumes                               |                                                  |
| `indexer.customVolumeMounts`                    | Custom Server volumeMounts                   |                                                  |
| `indexer.persistence.existingClaim`            | Provide an existing PersistentVolumeClaim    | `nil`                              |
| `indexer.persistence.storageClass`             | Storage class of backing PVC                 | `nil (uses default storage class annotation)`      |
| `indexer.persistence.enabled`                  | Xray Indexer persistence volume enabled      | `true`                             |
| `indexer.persistence.accessMode`               | Xray Indexer persistence volume access mode  | `ReadWriteOnce`                    |
| `indexer.persistence.size`                     | Xray Indexer persistence volume size         | `50Gi`                             |
| `indexer.resources`                            | Xray Indexer resources                       | `{}`                 |
| `indexer.preStartCommand`                      | Xray Indexer Custom command to run before startup. Runs AFTER the `common.preStartCommand` |     |
| `indexer.loggers`                              | Xray Indexer loggers (see values.yaml for possible values)   | ` `  |
| `indexer.loggersResources.requests.memory`     | Xray Indexer loggers initial memory request                                 |               |
| `indexer.loggersResources.requests.cpu`        | Xray Indexer loggers initial cpu request                                    |               |
| `indexer.loggersResources.limits.memory`       | Xray Indexer loggers memory limit                                           |               |
| `indexer.loggersResources.limits.cpu`          | Xray Indexer loggers cpu limit                                              |               |
| `indexer.nodeSelector`                         | Xray Indexer node selector                   | `{}`                 |
| `indexer.affinity`                             | Xray Indexer node affinity                   | `{}`                 |
| `indexer.tolerations`                          | Xray Indexer node tolerations                | `[]`                 |
| `persist.name`                                 | Xray Persist name                            | `xray-persist`       |
| `persist.image`                                | Xray Persist container image                 | `docker.bintray.io/jfrog/xray-persist`  |
| `persist.annotations`                          | Xray Persist annotations                     | `{}`                               |
| `persist.replicaCount`                         | Xray Persist replica count                   | `1`                  |
| `persist.updateStrategy`                       | Xray Persist update strategy                 | `RollingUpdate`      |
| `persist.podManagementPolicy`                  | Xray Persist pod management policy           | `Parallel`           |
| `persist.internalPort`                         | Xray Persist internal port                   | `7003`               |
| `persist.externalPort`                         | Xray Persist external port                   | `7003`               |
| `persist.service.type`                         | Xray Persist service type                    | `ClusterIP`          |
| `persist.livenessProbe.enabled`               | Xray Persist liveness probe enabled                       | `true`                  |
| `persist.livenessProbe.path`                  | Xray Persist liveness probe path                       | `/debug/pprof/`                  |
| `persist.livenessProbe.initialDelaySeconds`   | Xray Persist delay before liveness probe is initiated                       | `90`                  |
| `persist.livenessProbe.periodSeconds`         | Xray Persist how often to perform the probe                       | `10`                  |
| `persist.livenessProbe.timeoutSeconds`        | Xray Persist when the probe times out                       | `1`                  |
| `persist.livenessProbe.failureThreshold`      | Xray Persist minimum consecutive failures for the probe to be considered failed after having succeeded        | `3`           |
| `persist.livenessProbe.successThreshold`      | Xray Persist minimum consecutive successes for the probe to be considered successful after having failed | `1`                  |
| `persist.readinessProbe.enabled`               | Xray Persist readiness probe enabled                       | `true`                  |
| `persist.readinessProbe.path`                  | Xray Persist readiness probe path                       | `/debug/pprof/`                  |
| `persist.readinessProbe.initialDelaySeconds`   | Xray Persist delay before readiness probe is initiated                       | `90`                  |
| `persist.readinessProbe.periodSeconds`         | Xray Persist how often to perform the probe                       | `10`                  |
| `persist.readinessProbe.timeoutSeconds`        | Xray Persist when the probe times out                       | `1`                  |
| `persist.readinessProbe.failureThreshold`      | Xray Persist minimum consecutive failures for the probe to be considered failed after having succeeded        | `3`           |
| `persist.readinessProbe.successThreshold`      | Xray Persist minimum consecutive successes for the probe to be considered successful after having failed | `1`                  |
| `persist.persistence.size`                     | Xray Persist storage size limit              | `10Gi`               |
| `persist.preStartCommand`                      | Xray Persist Custom command to run before startup. Runs AFTER the `common.preStartCommand` |     |
| `persist.loggers`                              | Xray Persist loggers (see values.yaml for possible values)  | ` `   |
| `persist.loggersResources.requests.memory`     | Xray Persist loggers initial memory request                                 |               |
| `persist.loggersResources.requests.cpu`        | Xray Persist loggers initial cpu request                                    |               |
| `persist.loggersResources.limits.memory`       | Xray Persist loggers memory limit                                           |               |
| `persist.loggersResources.limits.cpu`          | Xray Persist loggers cpu limit                                              |               |
| `persist.resources`                            | Xray Persist resources                       | `{}`                 |
| `persist.nodeSelector`                         | Xray Persist node selector                   | `{}`                 |
| `persist.affinity`                             | Xray Persist node affinity                   | `{}`                 |
| `persist.tolerations`                          | Xray Persist node tolerations                | `[]`                 |
| `server.name`                                  | Xray server name                             | `xray-server`        |
| `server.image`                                 | Xray server container image                  | `docker.bintray.io/jfrog/xray-server`   |
| `server.annotations`                           | Xray server annotations                      | `{}`                               |
| `server.customVolumes`                         | Custom volumes                               |                                                  |
| `server.customVolumeMounts`                    | Custom Server volumeMounts                   |                                                  |
| `server.replicaCount`                          | Xray server replica count                    | `1`                  |
| `server.updateStrategy`                        | Xray server update strategy                  | `RollingUpdate`      |
| `server.podManagementPolicy`                   | Xray server pod management policy            | `Parallel`           |
| `server.internalPort`                          | Xray server internal port                    | `8000`               |
| `server.externalPort`                          | Xray server external port                    | `80`                 |
| `server.service.name`                          | Xray server service name                     | `xray`               |
| `server.service.type`                          | Xray server service type                     | `LoadBalancer`       |
| `server.service.annotations`                   | Xray server service annotations              | `{}`           |
| `server.livenessProbe.enabled`               | Xray server liveness probe enabled                       | `true`                  |
| `server.livenessProbe.path`                  | Xray server liveness probe path                       | `/web/`                  |
| `server.livenessProbe.initialDelaySeconds`   | Xray server delay before liveness probe is initiated                       | `90`                  |
| `server.livenessProbe.periodSeconds`         | Xray server how often to perform the probe                       | `10`                  |
| `server.livenessProbe.timeoutSeconds`        | Xray server when the probe times out                       | `1`                  |
| `server.livenessProbe.failureThreshold`      | Xray server minimum consecutive failures for the probe to be considered failed after having succeeded        | `3`           |
| `server.livenessProbe.successThreshold`      | Xray server minimum consecutive successes for the probe to be considered successful after having failed | `1`                  |
| `server.readinessProbe.enabled`               | Xray server readiness probe enabled                       | `true`                  |
| `server.readinessProbe.path`                  | Xray server readiness probe path                       | `/web/`                  |
| `server.readinessProbe.initialDelaySeconds`   | Xray server delay before readiness probe is initiated                       | `90`                  |
| `server.readinessProbe.periodSeconds`         | Xray server how often to perform the probe                       | `10`                  |
| `server.readinessProbe.timeoutSeconds`        | Xray server when the probe times out                       | `1`                  |
| `server.readinessProbe.failureThreshold`      | Xray server minimum consecutive failures for the probe to be considered failed after having succeeded        | `3`           |
| `server.readinessProbe.successThreshold`      | Xray server minimum consecutive successes for the probe to be considered successful after having failed | `1`                  |
| `server.persistence.existingClaim`             | Provide an existing PersistentVolumeClaim    | `nil`                              |
| `server.persistence.storageClass`              | Storage class of backing PVC                 | `nil (uses default storage class annotation)`      |
| `server.persistence.enabled`                   | Xray server persistence volume enabled       | `true`                             |
| `server.persistence.accessMode`                | Xray server persistence volume access mode   | `ReadWriteOnce`                    |
| `server.persistence.size`                      | Xray server persistence volume size          | `50Gi`                             |
| `server.preStartCommand`                       | Xray server Custom command to run before startup. Runs AFTER the `common.preStartCommand` |     |
| `server.loggers`                               | Xray server loggers (see values.yaml for possible values)  | ` `    |
| `server.loggersResources.requests.memory`      | Xray server loggers initial memory request                          |             |
| `server.loggersResources.requests.cpu`         | Xray server loggers initial cpu request                             |             |
| `server.loggersResources.limits.memory`        | Xray server loggers memory limit                                    |             |
| `server.loggersResources.limits.cpu`           | Xray server loggers cpu limit                                       |             |
| `server.resources`                             | Xray server resources                        | `{}`                 |
| `server.nodeSelector`                          | Xray server node selector                    | `{}`                 |
| `server.affinity`                              | Xray server node affinity                    | `{}`                 |
| `server.tolerations`                           | Xray server node tolerations                 | `[]`                 |

Specify each parameter using the `--set key=value[,key=value]` argument to `helm install`.

### Ingress and TLS
To get Helm to create an ingress object with a hostname, add these two lines to your Helm command:
```bash
helm install --name xray \
  --set ingress.enabled=true \
  --set ingress.hosts[0]="xray.company.com" \
  --set server.service.type=NodePort \
  jfrog/xray
```

If your cluster allows automatic creation/retrieval of TLS certificates (e.g. [cert-manager](https://github.com/jetstack/cert-manager)), please refer to the documentation for that mechanism.

To manually configure TLS, first create/retrieve a key & certificate pair for the address(es) you wish to protect. Then create a TLS secret in the namespace:

```bash
kubectl create secret tls xray-tls --cert=path/to/tls.cert --key=path/to/tls.key
```

Include the secret's name, along with the desired hostnames, in the Xray Ingress TLS section of your custom `values.yaml` file:

```yaml
  ingress:
    ## If true, Xray Ingress will be created
    ##
    enabled: true

    ## Xray Ingress hostnames
    ## Must be provided if Ingress is enabled
    ##
    hosts:
      - xray.domain.com
    annotations:
      kubernetes.io/tls-acme: "true"
    ## Xray Ingress TLS configuration
    ## Secrets must be manually created in the namespace
    ##
    tls:
      - secretName: xray-tls
        hosts:
          - xray.domain.com
```

### Ingress additional rules

You have the option to add additional ingress rules to the Xray ingress. An example for this use case can be routing the /artifactory path to Artifactory.
In order to do that, simply add the following to a `xray-values.yaml` file:
```yaml
ingress:
  enabled: true

  defaultBackend:
    enabled: false

  annotations:
    kubernetes.io/ingress.class: nginx
    nginx.ingress.kubernetes.io/configuration-snippet: |
      rewrite "(?i)/xray(/|$)(.*)" /$2 break;

  additionalRules: |
    - host: <MY_HOSTNAME>
      http:
        paths:
          - path: /
            backend:
              serviceName: {{ template "xray-server.fullname" . }}
              servicePort: {{ .Values.server.externalPort }}
          - path: /xray
            backend:
              serviceName: {{ template "xray-server.fullname" . }}
              servicePort: {{ .Values.server.externalPort }}
          - path: /artifactory
            backend:
              serviceName: <ARTIFACTORY_SERVICE_NAME>
              servicePort: <ARTIFACTORY_SERVICE_PORT>
``` 

and running:
```bash
helm upgrade --install xray jfrog/xray -f xray-values.yaml
```

### Custom volumes
If you need to use a custom volume in a custom init or sidecar container, you can use this option.

For this, there is a section for defining custom volumes in the [values.yaml](values.yaml). By default it's commented out
```yaml
server:
  ## Add custom volumes
  customVolumes: |
    ## Custom volume comes here ##
```

## Useful links
- https://www.jfrog.com/confluence/display/XRAY/Xray+High+Availability
- https://www.jfrog.com/confluence/display/EP/Getting+Started
- https://www.jfrog.com/confluence/
